
package xelfi.browser.document;

import xelfi.browser.view.MemberView;
import xelfi.editor.*;
import java.awt.Frame;
import java.io.IOException;
import java.io.DataInputStream;
import java.io.DataOutputStream;

public class BrMember extends BrComponent
{
    protected MyPosition  positionBegin;
    protected MyPosition  positionEnd;
    protected MyPosition  positionBodyEnd = null;
    protected MyPosition  commentPositionB;
    protected MyPosition  commentPositionE;
    /**
     * This variable points to the object which contains
     * this member.
     */
    public BrObject     object;
    /**
     * This variable contains representation of file name
     * of this member.
     */
    private XFile       xFile;
    /**
     * This variable stores the flags of this members.Flags
     * representates modifiers of this members.
     */
    public String       flags;
    /**
     * This variable stores information about type
     * of this object.
     */
    public String       type;
    /**
     * This variable stores the informations about the modifiers
     * of this member components.
     */
     public Modifiers    modifiers;

    /**
     * Constructor.
     * @param MyPosition pPositionB Begin of definition of this member.
     * @param MyPosition pPositionE End of definition of this member.
     * @param MyPosition pCommentPositionB Begin of java-doc comment
     *  of this member.
     * @param MyPosition pCommentPositionE End of java-doc comment
     *  of this member.
     * @param BrObject pObject Object which contains this member.
     * @param String pName Name of member.
     * @param XFile pXFile File in which is stored the object which contines
     *  this member.
     * @param Modifiers modif Modifiers of this member.
     * @param String pType Type of this member.
     */
    public BrMember(
                MyPosition pPositionB,
                MyPosition pPositionE,
                MyPosition pCommentPositionB,
                MyPosition pCommentPositionE,
                BrObject    pObject,
                String      pName,
                XFile      pXFile,
                Modifiers   modif,
                String      pType)
    {
        positionBegin   = pPositionB;
        positionEnd     = pPositionE;
        commentPositionB = pCommentPositionB;
        commentPositionE = pCommentPositionE;
        object          = pObject;
        name            = pName;
        xFile           = pXFile;
        modifiers       = modif;
        type            = pType;
        return;
    }
    BrMember(BrObject obj,DataInputStream is) throws IOException
    {
        positionBegin    = MyPosition.read(is);
        positionEnd      = MyPosition.read(is);
        positionBodyEnd  = MyPosition.read(is);
        commentPositionB = MyPosition.read(is);
        commentPositionE = MyPosition.read(is);
        object           = obj;
        name             = is.readUTF();
        xFile            = obj.getXFile();
        modifiers        = new Modifiers(is.readChar());
        type             = is.readUTF();
    }
    void save(DataOutputStream os) throws EndOfTextException,
        IOException
    {
        MyPosition.write(positionBegin,os);
        MyPosition.write(positionEnd,os);
        MyPosition.write(positionBodyEnd,os);
        MyPosition.write(commentPositionB,os);
        MyPosition.write(commentPositionE,os);
        os.writeUTF(name);
        os.writeChar(modifiers.getInt());
        os.writeUTF(type);
    }

    void open(Document d)
    {
        positionBegin.open(d);
        positionEnd.open(d);
        if( commentPositionB != null )
        {
            commentPositionB.open(d);
            commentPositionE.open(d);
        }
    }
    void close()
    {
        positionBegin.close();
        positionEnd.close();
        if( commentPositionB != null )
        {
            commentPositionB.close();
            commentPositionE.close();
        }
    }
    /**
     * This method sets position of the end of the body of this
     * member.
     * @param MyPosition position
     */
    public void setBodyEnd(MyPosition position)
    {
        positionBodyEnd = position;
    }
    /**
     * This method sets position of the end of the definition of this
     * member.
     * @param MyPosition position
     */
    public void setEnd(MyPosition position)
    {
        positionEnd = position;
    }
    /**
     * This method sets position of the begin of the definition of this
     * member.
     * @param MyPosition position
     */
    public void setBegin(MyPosition position)
    {
        positionBegin = position;
    }
    /**
     * This method returns position of the begin of the definition of this
     * member.
     * @return MyPosition position
     */
    public Position getBegin()
    {
        return positionBegin.getPosition();
    }
    /**
     * This method returns position of the end of the definition of this
     * member.
     * @return MyPosition position
     */
    public Position getEnd()
    {
        return positionEnd.getPosition();
    }
    /**
     * This method returns position of the end of the body of this
     * member.
     * @return MyPosition position
     */
    public Position getBodyEnd()
    {
        return positionBodyEnd.getPosition();
    }
    /**
     * This method generates source text of this method.
     * @return The text.
     */
    public String getText()
    {
        return "";
    }
    /**
     * This method opens view for this member.
     * @param f Parent frame
     */
    public void openView(Frame f)
    {
        MemberView v = new MemberView(f,this);
    }
    /**
     * This method returns true, if selecting of this component
     * is allowed.
     * @return true
     */
    public boolean selectable()
    {
        return true;
    }
    public XFile getXFile()
    {
        return xFile;
    }
    public void deleteMember()
        throws IOException,InvalidPositionException,NotAllowedException,
            EndOfTextException
    {
        Block b;
        Source s = new Source(xFile);
        (new Block(getBegin(),getEnd())).delete();
        if( commentPositionB != null )
            (new Block(commentPositionB.getPosition().previous(3),
                commentPositionE.getPosition())).delete();
        object.removeComponent(this);
        return;
    }
    /**
     * This method allows adding a new member behind
     * this one. It generates source code too.
     *
     * @param newMember Member to add.
     * @exception IOException
     * @exception NotAllowedException Occures, when you adds text
     *  to the form.
     * @exception EndOfText Exception
     */
    public void addMember(BrMember newMember)
        throws IOException,NotAllowedException,EndOfTextException
    {
    }
    public void changeMember(BrMember newMember)
        throws IOException,NotAllowedException,EndOfTextException
    {
    }
    public String getHint()
        throws IOException
    {
        return "";
    }
    public void setHint(String hint)
        throws IOException,NotAllowedException
    {
        if( commentPositionB == null )
        {
            int i,indent;
            hint = strToHint(hint,indent = positionBegin.getPosition().
                getColumn());
            Position p,pom;
            Source s = new Source(getXFile());
            pom = getBegin();
            p = pom.insert("/**\n" + hint);

            StringBuffer b = new StringBuffer();
            b.append('\n');
            for(i=0;i<indent;i++) b.append(' ');
            commentPositionE = new MyPosition(
                pom.insert(new String(b)));

            commentPositionB = new MyPosition(p.next(3));
        }else
        {
            hint = strToHint(hint,commentPositionB.getPosition().
                getColumn()-2);
            Position p,pom;
            Source s = new Source(getXFile());
            (new Block(pom = commentPositionB.getPosition(),
                commentPositionE.getPosition())).delete();
            p = pom.insert("\n" + hint);

            commentPositionB = new MyPosition(p);
            commentPositionE = new MyPosition(pom);
        }
        return;
    }
}







