/**
* Xelfi programming environment
*/

package xelfi.browser.document;

import xelfi.browser.view.BrowserPanel;
import xelfi.browser.view.PackageView;
import xelfi.browser.util.Sort;
import java.util.*;
import java.io.*;
import java.awt.*;
import xelfi.editor.EndOfTextException;
/**
* This class
*
* @version 1.0
* @author Jan Jancura
*/
public class BrPackage extends BrComponent
{
    private XFile       xFile;
    private int         project;        // Project,system,non

    private boolean     opened          = false;
    private boolean     root            = false;
    private int         classNum        = 0;
    private int         interfaceNum    = 0;
    private Vector      member          = new Vector(10,3);

   /**
    * This method
    *
    * @param
    * @return
    * @exception
    */
    public BrPackage(XFile pFile,int pProject)
    {
        xFile = pFile;
        name = xFile.getPackage();
        if( name.equals("") )
            name = "# " + xFile.getPath();
        project = pProject;
        member = new Vector(40,20);
    }

   /**
    * This method
    *
    * @param
    * @return
    * @exception
    */
    public int getProject()
    {
        return project;
    }

   /**
    * This method
    *
    * @param
    * @return
    * @exception
    */
    public void setProject(int p)
    {
        project = p;
    }

   /**
    * This method
    *
    * @param
    * @return
    * @exception
    */
    public XFile getXFile()
    {
        return xFile;
    }

   /**
    * This method
    *
    * @param
    * @return
    * @exception
    */
    public boolean addComponent(BrComponent c)
    {
        member.addElement(c);
        if( c instanceof BrClass )
            classNum++;
        else
        if( c instanceof BrInterface )
            interfaceNum++;
        else
            System.out.println("BrPackage.addComponent snazim se pridat cosi divneho!!! :" + c);
        return true;
    }
    private void Open()
        throws FileNotFoundException,IOException
    {
        opened = true;
        ComponentManager.parsePackage(this);
        return;
    }
    void save()
    {
        if( !opened ) return;

        //System.out.println("BrPackage.save " + getXFile().getPackage());
        File f = new File(getXFile().getFile(),getXFile().
            getPackage() + ".brw");
        try{
            DataOutputStream os = new DataOutputStream(new
                FileOutputStream(f));
            BrObject o;
            int i,k = member.size();
            for(i=0;i<k;i++)
            {
                o = (BrObject)member.elementAt(i);
                if( o instanceof BrInterface )
                {
                    os.writeByte('i');
                    o.save(os);
                }else
                if( o instanceof BrClass )
                {
                    if( ((BrClass)o).isForm )
                        os.writeByte('f');
                    else
                        os.writeByte('c');
                    o.save(os);
                }
            }//for
            os.writeByte('e');
        }catch(IOException e)
        {
            System.out.println("BrPackage save() IOException");
        }catch(EndOfTextException e)
        {
            System.out.println("BrPackage save() IOException");
        }
        //System.out.println("BrPackage.save end");
        return;
    }
   /**
    * This method
    *
    * @param
    * @return
    * @exception
    */
    public Enumeration getAllObjects()
    {
        return member.elements();
    }

   /**
    * This method
    *
    * @param
    * @return
    * @exception
    */
    public BrObject getObject(String pName)
    {
        BrObject o;
        for(Enumeration e = member.elements();
                e.hasMoreElements();)
            if( (o = (BrObject)e.nextElement()).getKey().equals(pName) )
                return o;
        return null;
    }

   /**
    * This method
    *
    * @param
    * @return
    * @exception
    */
    public void removeObject(String pName)
    {
        BrObject o = getObject(pName);
        if( o != null )
        {
            member.removeElement(o);
            if( o instanceof BrClass )
                classNum--;
            else
            if( o instanceof BrInterface )
                interfaceNum--;
        }
        return;
    }

   /**
    * This method
    *
    * @param
    * @return
    * @exception
    */
    public void removeObject(BrObject o)
    {
        if( member.removeElement(o) )
        {
            if( o instanceof BrClass )
                classNum--;
            else
            if( o instanceof BrInterface )
                interfaceNum--;
        }
        return;
    }

   /**
    * This method
    *
    * @param
    * @return
    * @exception
    */
    public void removeObjects()
    {
        member.removeAllElements();
        classNum = 0;
        interfaceNum = 0;
        return;
    }

   /**
     * This method returns a number of actually visible
     * subcomponents.
     * @param bs Actuall settings
     * @return Number of visible subcomponents
     */
    public int getComponentsNumber(BrSettings bs)
       throws FileNotFoundException,IOException
    {
        if( !opened ) Open();
        int i = 0;
        if( bs.MultipleSelectioned && (bs.ObjSort == 0) )
            i = 1;
        return (bs.ObjClass?classNum:0) +
               (bs.ObjInterface?interfaceNum:0) + i;
    }

   /**
     * This method returns actually visible subcomponents (classes
     * and interfaces) of this component. This components are stored
     * to the given array form @see from index.
     * @param bs Actual settings.
     * @param c The array of the components.
     * @param from First non occupied position.
     * @return The index of the first non occupied position
     * past adding components to the array
     */
    public int getComponents(BrSettings bs,BrComponent[] c,int from)
        throws FileNotFoundException,IOException
    {
        if( !opened ) Open();

        if( bs.MultipleSelectioned && (bs.ObjSort == 0) )
            c[from++] = new BrComponent(name);

        BrComponent p;
        int i,od = from,k = member.size();
        for(i=0;i<k;i++)
        {
            p = (BrComponent)member.elementAt(i);
            if( bs.ObjClass && (p instanceof BrClass) )
                c[from++] = p;
            else
            if( bs.ObjInterface && (p instanceof BrInterface) )
                c[from++] = p;
        }
        if( bs.ObjSort == 0 )
            c = (BrComponent[])Sort.sort(c,od,from);

        return from;
    }

   /**
     * This method opens view for this package.
     * @param f Parent frame
     */
    public void openView(Frame f)
    {
        PackageView v = new PackageView(f,this);
    }

   /**
     * This method returns true, if selecting of this component
     * is allowed.
     * @return true
     */
    public boolean selectable()
    {
        return true;
    }

}














