
package xelfi.browser.document;

import java.util.Date;
import java.io.IOException;
import java.io.InputStream;
import xelfi.editor.*;
import java.io.DataInputStream;
import java.io.DataOutputStream;

public class BrVariable extends BrMember
{
    public String       init;
    public boolean      one;

    public BrVariable (
                MyPosition pPositionB,
                MyPosition pPositionE,
                MyPosition pCommentPositionB,
                MyPosition pCommentPositionE,
                boolean     pOne,
                BrObject    pObject,
                String      pName,
                XFile       pXFile,
                Modifiers   modif,
                String      pType,

                String      pInit)
    {
        super(pPositionB,pPositionE,pCommentPositionB,pCommentPositionE,
            pObject,pName,pXFile,modif,pType);
        init = pInit.trim(); //(pInit.length()>0 ? " = " : "") + pInit;
        if( init.startsWith("=") )
            init = init.substring(1).trim();

        one = pOne;
        initFlags();
    }
    private void initFlags()
    {
        StringBuffer buf = new StringBuffer();
        buf.append(modifiers.finalM() ? 'f' : '-');
        buf.append(modifiers.threadsafeM() ? "Th" : "--");
        buf.append(modifiers.staticM() ? "St" : "--");
        buf.append(modifiers.transientM() ? "Tr" : "--");
        buf.append(modifiers.access());
        buf.append(" ");
        flags = new String(buf);
        return;
    }
    BrVariable(BrObject obj,DataInputStream is)
        throws IOException
    {
        super(obj,is);
        init = is.readUTF();
        one = is.readBoolean();
        initFlags();
        return;
    }
    void save(DataOutputStream os) throws EndOfTextException,
        IOException
    {
        super.save(os);
        os.writeUTF(init);
        os.writeBoolean(one);
        return;
    }
    /**
     * This method return name with modifiers and other
     * parameters.
     * @param bs Actual settings.
     * @param depth If this component listbox is hierarched
     * this parameter stores information about components depth
     * in this tree.
     * @return Full name
     */
    public String getFullName(BrSettings bs,int depth)
    {
        StringBuffer s = new StringBuffer();
        if( bs.MemShowModif )
        {
            s.append(flags);
        }
        if( bs.MemShowType )
        {
            s.append(type);
            int i,k = bs.MemIndent - type.length();
            k = k > 0 ? k : 1;
            for(i=0;i<k;i++)
                s.append(' ');
        }
        s.append(name);
        if( bs.MemShowInit )
            if( init.length() > 0 )
                s.append(" = ").append(init);
        return new String(s);
    }
    public String getText()
    {
        StringBuffer b = new StringBuffer();
        if( one )
        {
            switch( modifiers.access() )
            {
            case 0: b.append("public ");break;
            case 2: b.append("private ");break;
            case 3: b.append("protected ");break;
            }
            b.append(modifiers.staticM() ? "static " : "").
            append(modifiers.finalM() ? "final " : "").
            append(modifiers.threadsafeM() ? "threadsafe " : "").
            append(modifiers.transientM() ? "transient " : "").

            append(type).
            append(" ").
            append(name);
            if( init.length() > 0 )
                b.append(" = ").append(init);

            b.append(';');
        }else
        {
            b.append(name);
            if( init.length() > 0 )
                b.append(" = ").append(init);
        }
        return new String(b);
    }
    /**
     * This method allows adding a new member behind
     * this one. It generates source code too.
     * @param newMember Member to add.
     * @exception IOException
     * @exception NotAllowedException Occures, when you adds text
     * to the form.
     * @exception EndOfText Exception
     */
    public void addMember(BrMember newVariable,boolean pNew)
        throws IOException,NotAllowedException
    {
        Position p,pom;
        Source s = new Source(getXFile());
        pom = getEnd();
        if( one )
            p = pom.insert("\n    " +
                newVariable.getText());
        else
            p = pom.insert("," + newVariable.getText());
        object.addComponentBehind(newVariable,this);
        if( one )
            newVariable.setBegin(new MyPosition(p.next(5)));
        else
            newVariable.setBegin(new MyPosition(p.next()));
        newVariable.setEnd(new MyPosition(pom));
        setEnd(new MyPosition(p));
        return;
    }
    public void changeMember(BrMember newVariable)
        throws IOException,NotAllowedException
    {
        Position p,pom;
        Source s = new Source(getXFile());
        (new Block(getBegin(),getEnd())).delete();
        pom = getEnd();
        if( one )
            p = pom.insert(newVariable.getText());
        else
            p = pom.insert(newVariable.getText());

        object.addComponentBehind(newVariable,this);
        object.removeComponent(this);

        newVariable.setBegin(new MyPosition(p));
        newVariable.setEnd(new MyPosition(pom));
        setEnd(new MyPosition(p));
        return;
    }
    public String getHint()
        throws IOException
    {
        if( commentPositionB == null )
        {
            return "This variable \n";
        }else
        {
            Source src = new Source(getXFile());
            InputStream is = (new Block(commentPositionB.getPosition(),
                commentPositionE.getPosition())).
                getInputStream();
            StringBuffer b = new StringBuffer();
            int c;
            while( (c = is.read()) >= 0 ) b.append((char)c);
            String s = new String(b);

            return hintToString(s);
        }
    }
}









