/**
* Xelfi
*/

package xelfi.design.forms;

import java.util.Vector;
import java.util.Hashtable;
import java.util.Enumeration;
import xelfi.design.util.XFDesignOptions;

/**
* This class manages imports for one form.
* It manages list of all needed imports and provides method for generating imports.
* @see XelfiDesignComponent#neededImports
* @version 	1.01, 18 Feb 1996 
* @finished
* @author 		Ian Formanek
*/

class XFImportsManager {
	private Vector list;
	
	XFImportsManager() {
		list = new Vector(1,1);
	}
	
	private ImportInfo findImport(String aName) {
		int count = list.size();
		for (int i = 0; i<count; i++) {
			ImportInfo ii = (ImportInfo)list.elementAt(i);
			if (aName.compareTo(ii.name) == 0) 
				return ii;
		}
		return null;
	}
	
	void assignImports(String imps[]) {
		for (int i=0; i<imps.length; i++)
			assignImport(imps[i]);
	}
	
	void assignImport(String imp) {
		ImportInfo ii = findImport(imp);
		if (ii == null) ii = new ImportInfo(imp);
		else ii.refCount++;
		list.addElement(ii);
	}

	void releaseImports(String imps[]) {
		for (int i=0; i<imps.length; i++)
			releaseImport(imps[i]);
	}

	void releaseImport(String imp) {
		ImportInfo ii = findImport(imp);
		if (ii != null) {
			ii.refCount--;
			if (ii.refCount == 0)
				list.removeElement(ii);
		}
	}
	
	private String getPackageFromName(String aName) {
		int ind = aName.lastIndexOf(".");
		if (ind == -1) return aName;
		String str = aName.substring(0,ind);
		return aName.substring(0,ind);
	}
	
	private Vector makePackageList() {
		Hashtable countHash = new Hashtable(); // key = package; value = refCount
		Hashtable starredHash = new Hashtable();
		Vector countVect = new Vector(1,1);
		int count = list.size();
		for (int i=0; i<count; i++) {
			String wholeName = ((ImportInfo)list.elementAt(i)).name;
			String packName = getPackageFromName(wholeName);
		
			if (wholeName.endsWith(".*"))
				starredHash.put(packName,packName);
			else 
			if (starredHash.get(packName) == null) {
				PackInfo pi;
				if ((pi = (PackInfo)countHash.get(packName)) == null)
					countHash.put(packName, new PackInfo(wholeName));
				else 
					pi.acquire(wholeName);
			}
		}
		// now:
		// in starredHash we have all packages with stars at the end
		// in countHash we have pairs of <package name, packinfo>
		//   then if the count in packinfo > XFDesignOptions.importsThreshold make it a starred package
		//		else make it as normal packages
		Vector packages = new Vector(0,0);
		Enumeration countKeys = countHash.keys();
		while (countKeys.hasMoreElements()) { 
			String packKey = (String)countKeys.nextElement();
			PackInfo pi = (PackInfo)countHash.get(packKey);
			if (starredHash.get(packKey) == null)				
				if (pi.refCount > XFDesignOptions.importsThreshold)
					starredHash.put(packKey, packKey);
				else {
					count = pi.names.size();
					for (int i = 0; i<count; i++)
						packages.addElement(pi.names.elementAt(i));
				}
		}
		Enumeration starredKeys = starredHash.keys();
		while (starredKeys.hasMoreElements()) { 
			String starredKey = (String)starredKeys.nextElement();
			packages.insertElementAt(starredKey+".*",0);
		}
		return packages;
	}
	
	private String[] sortList(Vector vect) {
		int count = vect.size();
		String list[] = new String[count];
		for (int i=0; i<count; i++) {
			String min = (String)vect.elementAt(0);
			int minIdx = 0;
			for (int j=1; j<count-i;j++)
				if (min.compareTo((String)vect.elementAt(j))>0) {
					min = (String)vect.elementAt(j);
					minIdx = j;
				}
			list[i] = min;
			vect.removeElementAt(minIdx);
		}
		return list;
	}

	String[] getImports() {
		return sortList(makePackageList());
	}

	String generateImports() {
		String gen = "";
		String imps[] = getImports();
		for (int i = 0; i< imps.length; i++)
			gen = gen+"import "+ imps[i]+";\n";
		gen = gen + "\n";
		return gen;
	}
}

class ImportInfo {
	String name;
	int refCount;
	ImportInfo(String aName) {
		name = aName;
		refCount = 1;
	}
}

class PackInfo {
	int refCount;
	Vector names;
	PackInfo(String wholeName) {
		refCount = 1;
		names = new Vector(1,1);
		names.addElement(wholeName);
	}
	
	void acquire(String wholeName) {
		int count = names.size();
		for (int i=0; i<count; i++)
			if (wholeName.compareTo((String)names.elementAt(i)) == 0)
				return;
		refCount++;
		names.addElement(wholeName);
	}
}
	