/**
* Xelfi programming environment
*/

package xelfi.editor;
import java.awt.*;
/**
* Extension to scrollbar class that is designed to work on a document as a horizontal scrollbar.
*
* @version 1.0
* @author Jaroslav Tulach
*/

class EditorHScrollbar extends Scrollbar implements CursorListener {
  /** Max Line Size */
  private static final int MAX_LINE_SIZE = 128;
  /** The editor canvas */
  private TextEditorInfo editor;

  /** Constructs new scrollbar. Counts number of lines in document.
  * Adds itself as a controler to the document.
  * @param e the editor that this scrollbar belongs to
  */
  public EditorHScrollbar (TextEditorInfo e) {
    super (HORIZONTAL);
    editor = e;
    editor.addCursorListener (this);
    linesChanged ();
  };

  /** What to do on editor close.
  */
  public void close () {
    editor.removeCursorListener (this);
  };
  /** Sets correct values to the scrollbar.
  */
  public void linesChanged () {
    Position p = editor.getCursorPos ();
    int v = p.getColumn ();
    if (v > MAX_LINE_SIZE) v = MAX_LINE_SIZE;
    setValues (v, 1, 0, MAX_LINE_SIZE);
  };

  /** Handles scrollbar changes.
  * @param e the event
  */
  public boolean handleEvent (Event e) {
    Position p = editor.getCursorPos ();
    try {
      switch (e.id) {
      case Event.SCROLL_LINE_UP:
        editor.setCursorPos (p.previous ());
        break;
      case Event.SCROLL_LINE_DOWN:
        editor.setCursorPos (p.next ());
        break;
      case Event.SCROLL_PAGE_UP:
        editor.setCursorPos (p.beginOfLine ());
        break;
      case Event.SCROLL_PAGE_DOWN:
        editor.setCursorPos (p.endOfLine ());
        break;
      case Event.SCROLL_ABSOLUTE:
        editor.setCursorPos (p.column (getValue ()));
        break;
      default:
        return super.handleEvent(e);
      };
    } catch (EndOfTextException ex) {
    };
    linesChanged ();
    return true;
  };
  /** Called when the cursor moves.
  * @param the editor info
  */
  public void cursorMoved (TextEditorInfo info) {
    linesChanged ();
  }
};




