/**
*  Xelfi Environment
*/

package xelfi.editor.keys;
import xelfi.editor.*;
import java.util.Vector;

/** Every object that wants to react to key events must implements
* this interface. The object will be created when the input file
* is parsed.
* Additional parameters will be passed through method init.
* When appropriate key action occures the keyDown method is called.
* It can throw an exception to inditate that action is not possible.
*
* @warning Do not forget to make subclasses public and to provide public
*   default constructor. Otherwise it will not be possible to create
*   these classes dynamicly.
*
* @author Jaroslav Tulach
* @version 1.0
*/

public interface KeyActionHandler {
  /** Initialize the object.
  * @param v vector of parameters. It can contain objects of type
  *   String, Integer, Character.
  * @exception IllegalArgumentException if a parameter is not valid
  */
  public void init (Vector v);

  /** Reacts to a key.
  * @param key the key code that occured
  * @param modifiers the state of modifiers
  * @param info info about editor
  * @return true if next command should be excecuted, false if
  *   the action should stop
  * @exception NotAllowedException if the action is not allowed
  * @exception EndOfTextException if end of text is reached
  */
  public boolean keyDown (int key, int modifiers, TextEditorInfo info)
  throws NotAllowedException, EndOfTextException;

}












// Generated from substitution template 'Interface' Blocks:Class:17,22,33,22; Package:8,4,25,4; 
