package xelfi.execution;

import java.awt.Panel;
import java.io.*;

import xelfi.top.*;
import xelfi.util.*;

public class RunOptions implements XelfiOptions
{
    // RunOptionsPanel works directly with these variables
  static String
    ipath = initInterpretPath(),
    args = "",
    mclass = null;
  static boolean
    showIO = true,
    togetherOE = true,
    showIn = true,
    close = true;
  static int clearOutAfter = 2048, appletWidth = 400, appletHeight = 200;

  private static String initInterpretPath()
  {
    String tmp = PathScanner.addFileSeparator(System.getProperty("java.home"));
    return tmp + "bin" + File.separator + "java";
  }

  public static String getInterpretPath()
  {
    return ipath;
  }

  public static String getArguments()
  {
    return args;
  }

  public static String getMainClass()
  {
    return mclass;
  }

  public static boolean showIOWindow()
  {
    return showIO;
  }

  public static boolean stdOutErrTogether()
  {
    return togetherOE;
  }

  public static boolean showStdin()
  {
    return showIn;
  }

  public static boolean closeWhenFinished()
  {
    return close;
  }

  public static int clearOutputAfter()
  {
    return clearOutAfter;
  }

  /** Applet height. If iz lower than zero, applet viewer will use preferred size of the applet. */
  public static int getAppletHeight()
  {
    return appletHeight;
  }

  /** Applet height. If iz lower than zero, applet viewer will use preferred size of the applet. */
  public static int getAppletWidth()
  {
    return appletWidth;
  }

    /** Sets the main class property.
      @param mc new main class or null.
    */
  public static void setMainClass(String mc)
  {
    if(mc != null) mclass = mc.length() == 0 ? null : mc;
    else mclass = null;
    TopLevel.getMenu().enableItem(MainMenu.miRunMain, mclass != null);
    TopLevel.getMenu().enableItem(MainMenu.miCompileMain, mclass != null);
  }

    /** XelfiOptions implementation */
  public void storeProperties(EnhProperties dest)
  {
    if(mclass != null) dest.put("MainClass", mclass);
    dest.put("JavaInterpreterPath", ipath);
    dest.put("RunArguments", args);
    dest.setBooleanProperty("ShowStdIOWindow", showIO);
    dest.setBooleanProperty("StErrOutTogether", togetherOE);
    dest.setBooleanProperty("ShowStdIn", showIn);
    dest.setBooleanProperty("CloseWhenFinished", close);
    dest.setIntProperty("ClearProcessOutputAfter", clearOutAfter);
    dest.setIntProperty("AppletInitialWidth", appletWidth);
    dest.setIntProperty("AppletInitialHeight", appletHeight);
  }

  public void loadProperties(EnhProperties src)
  {
    setMainClass((String)src.get("MainClass"));
    String tmp = (String)src.get("JavaInterpreterPath");
    ipath = tmp == null ? ipath : tmp;
    tmp = (String)src.get("RunArguments");
    args =  tmp == null ? args : tmp;
    showIO = src.getBooleanProperty("ShowStdIOWindow", showIO);
    togetherOE = src.getBooleanProperty("StErrOutTogether", togetherOE);
    showIn = src.getBooleanProperty("ShowStdIn", showIn);
    close =  src.getBooleanProperty("CloseWhenFinished", close);
    clearOutAfter = src.getIntProperty("ClearProcessOutputAfter", clearOutAfter);
    appletWidth = src.getIntProperty("AppletInitialWidth", appletWidth);
    appletHeight = src.getIntProperty("AppletInitialHeigt", appletHeight);
  }

  public Panel getPanel()
  {
    return new RunOptionsPanel();
  }

    /** XelfiOptions implementation */
  public String getTabName()
  {
    return "Run";
  }

    /** XelfiOptions implementation */
  public void onOK(Panel p)
  {
    ((RunOptionsPanel)p).saveValues();
  }

}






