/** Xelfi Main window
* @author: Filip Dvorak
* @version: 1.3.2 // improved by Ian on Feb,27 - reaction on ComponentsOrder, showGrid and lockComponents
*/

package xelfi.top;

import java.awt.*;
import java.io.*;
import java.util.*;

import xelfi.top.visual.*;
import xelfi.top.visual.EditorMenuItem;
import xelfi.compiler.*;
import xelfi.execution.*;

import xelfi.debugger.*;
import xelfi.browser.view.*;

import xelfi.design.components.palette.*;
import xelfi.design.components.dialogs.*;
import xelfi.design.components.XelfiVisualContainer;

import xelfi.design.forms.*;
import xelfi.design.util.XFDesignOptions;

import xelfi.design.inspector.*;
import xelfi.top.toolbar.*;

import xelfi.editor.*;
import xelfi.xedit.*;
import xelfi.util.*;

  /** The xelfi main window. */
public class MainMenu extends TopFrame
{
    /** Public symbolic constatns for both menu items and submenus. Items starts with "mi".
      First part of the item's symbolic name is often the name of it's parent submenu (but not always).
     */
  public static final int
      // First level menus
    systemMenu = 1,
    editMenu = 2,
    browseMenu = 3,
    designMenu = 4,
    runMenu = 5,
    debugMenu = 6,
    windowMenu = 7,
    helpMenu = 8,
      // Second level menus
    packageMenu = 9,
    classMenu = 10,
    methodMenu = 11,
    addMenu = 12,
      // all menu items
    miSystemNewBrowser = 65,
    miSystemNewClass = 13,
    miSystemNewPackage = 68,
    miSystemSaveTemplate = 16,
    miSystemSaveAll = 17,
    miSystemOptions = 58,
    miSystemExit = 18,
    miSystemSave = 14,
    miSystemMemory = 54,
    miEditUndo = 19,
    miEditCut = 20,
    miEditCopy = 21,
    miEditPaste = 22,
    miEditFind = 75,
    miEditGoTo = 76,
    miDesignNewForm = 36,
    miDesignSelectionMode = 37,
    miDesignPaletteManager = 70,
    miDesignShowGrid = 71,
    miDesignLockComponents = 77,
    miDesignDock = 69,
    miDesignComponentsOrder = 55,
    miDesignRealMode = 72,
    miCompile = 42,
    miCompileMain = 43,
    miCompileAll = 51,
    miRun = 44,
    miRunMain = 45,
    miRunApplet = 78,
    miRunSetMain = 73,
    miDebugGo = 46,
    miDebugGoCursor = 47,
    miDebugFinish = 48,
    miDebugRestart = 67,
    miDebugTraceOver = 49,
    miDebugTraceInto = 50,
    miDebugToggleBreak = 52,
    miDebugAddWatch = 53,
    miWindowClose = 56,
    miWindowCloseAll = 57,
    miWindowWatches = 59,
    miWindowLocals = 60,
    miWindowCallStack = 61,
    miWindowThreads = 62,
    miWindowBreakpoints = 63,
    miWindowOutput = 64,
    miWindowInspector = 74,
    miHelpAbout = 66;

  /* Note: first index: 1,
           last idex: 78
           holes in the region:  none
  */

  private MenuItem items[];

  MainMenu()
  {
    super("Xelfi");

      // to easier access the submenu objects:
    Menu tmpSystemMenu, tmpeditMenu, tmpbrowseMenu, tmpdesignMenu, tmprunMenu, tmpdebugMenu, tmpwindowMenu, tmphelpMenu,
      tmppackageMenu, tmpclassMenu, tmpmethodMenu, tmpaddMenu;

    setLayout(new BorderLayout());
    items = new MenuItem[80];

    MenuBar mb = new MenuBar();

    mb.add((Menu)(items[systemMenu] =  tmpSystemMenu = new Menu(translate("&System"))));
    tmpSystemMenu.add(items[miSystemNewClass] = new MenuItem(translate("New &class")));
    tmpSystemMenu.add(items[miSystemNewPackage] = new MenuItem(translate("New &package")));
    tmpSystemMenu.add(items[miSystemNewBrowser] = new MenuItem(translate("New b&rowser")));
    tmpSystemMenu.addSeparator();
    tmpSystemMenu.add(items[miSystemSave] = new MenuItem(translate("&Save")));
    tmpSystemMenu.add(items[miSystemSaveAll] = new MenuItem(translate("Save al&l")));
    tmpSystemMenu.add(items[miSystemSaveTemplate] = new MenuItem(translate("Save as &template")));
    tmpSystemMenu.addSeparator();
    tmpSystemMenu.add(items[miSystemOptions] = new MenuItem(translate("O&ptions")));
    tmpSystemMenu.add(items[miSystemMemory] = new MenuItem(translate("&Memory view")));
    tmpSystemMenu.addSeparator();
    tmpSystemMenu.add(items[miSystemExit] = new MenuItem(translate("E&xit IDE    Ctrl+X")));

    mb.add((Menu)(items[editMenu] = tmpeditMenu = new Menu(translate("&Edit"))));
    tmpeditMenu.add(items[miEditUndo] = new MenuItem(translate("&Undo")));
    tmpeditMenu.addSeparator();
    tmpeditMenu.add(items[miEditCut] = new MenuItem(translate("Cu&t")));
    tmpeditMenu.add(items[miEditCopy] = new MenuItem(translate("&Copy")));
    tmpeditMenu.add(items[miEditPaste] = new MenuItem(translate("&Paste")));
    tmpeditMenu.addSeparator();
    tmpeditMenu.add(items[miEditFind] = new MenuItem(translate("&Find and replace")));
    tmpeditMenu.add(items[miEditGoTo] = new MenuItem(translate("&Go to line")));


    mb.add((Menu)(items[designMenu] = tmpdesignMenu = new Menu(translate("&Design"))));
    tmpdesignMenu.add(items[addMenu] = tmpaddMenu = new Menu(translate("&Add Component")));
    tmpdesignMenu.add(items[miDesignSelectionMode] = new CheckboxMenuItem(translate("&Selection Mode")));
    tmpdesignMenu.addSeparator();
    tmpdesignMenu.add(items[miDesignPaletteManager] = new MenuItem(translate("&Palette manager...")));
    tmpdesignMenu.add(items[miDesignDock] = new CheckboxMenuItem(translate("&Docked palette")));
    tmpdesignMenu.addSeparator();
    tmpdesignMenu.add(items[miDesignShowGrid] = new CheckboxMenuItem(translate("Show &grid")));
    tmpdesignMenu.add(items[miDesignLockComponents] = new CheckboxMenuItem(translate("&Lock components")));
    tmpdesignMenu.add(items[miDesignComponentsOrder] = new MenuItem(translate("Components &order...")));
    tmpdesignMenu.add(items[miDesignRealMode] = new CheckboxMenuItem(translate("&Real mode")));

    mb.add((Menu)(items[runMenu] = tmprunMenu = new Menu(translate("&Run"))));
    tmprunMenu.add(items[miCompile] = new MenuItem(translate("&Compile class")));
    tmprunMenu.add(items[miCompileMain] = new MenuItem(translate("Compile &main class")));
    tmprunMenu.add(items[miCompileAll] = new MenuItem(translate("Compile &all")));
    tmprunMenu.addSeparator();
    tmprunMenu.add(items[miRunMain] = new MenuItem(translate("&Run main class")));
    tmprunMenu.add(items[miRun] = new MenuItem(translate("Run &class")));
    tmprunMenu.add(items[miRunApplet] = new MenuItem(translate("Run as &Applet")));
    tmprunMenu.addSeparator();
    tmprunMenu.add(items[miRunSetMain] = new MenuItem(translate("&Set as main")));

    mb.add((Menu)(items[debugMenu] = tmpdebugMenu = new Menu(translate("&Debug"))));
    tmpdebugMenu.add(items[miDebugGo] =          new MenuItem(translate("&Go                         F5")));
    tmpdebugMenu.add(items[miDebugGoCursor] =    new MenuItem(translate("Go to &cursor               F4")));
    tmpdebugMenu.add(items[miDebugFinish] =      new MenuItem(translate("&Finish debugging      Ctrl+F5")));
    tmpdebugMenu.add(items[miDebugRestart] =     new MenuItem(translate("&Restart debugging    Shift+F5")));
    tmpdebugMenu.addSeparator();
    tmpdebugMenu.add(items[miDebugTraceOver] =   new MenuItem(translate("&Trace over                 F8")));
    tmpdebugMenu.add(items[miDebugTraceInto] =   new MenuItem(translate("Trace &into                 F7")));
    tmpdebugMenu.addSeparator();
    tmpdebugMenu.add(items[miDebugToggleBreak] = new MenuItem(translate("Toggle &breakpoint          F9")));
    tmpdebugMenu.add(items[miDebugAddWatch] =    new MenuItem(translate("&Add watch             Ctrl+F9")));

    mb.add((Menu)(items[windowMenu] = tmpwindowMenu = new Menu(translate("&Window"))));
    tmpwindowMenu.add(items[miWindowClose] = new MenuItem(translate("&Close")));
    tmpwindowMenu.add(items[miWindowCloseAll] = new MenuItem(translate("Close &all")));
    tmpwindowMenu.addSeparator();
    tmpwindowMenu.add(items[miWindowWatches] =     new CheckboxMenuItem(translate("&Watches             Ctrl+Shift+W")));
    tmpwindowMenu.add(items[miWindowLocals] =      new CheckboxMenuItem(translate("&Locals              Ctrl+Shift+L")));
    tmpwindowMenu.add(items[miWindowCallStack] =   new CheckboxMenuItem(translate("Call &stack          Ctrl+Shift+S")));
    tmpwindowMenu.add(items[miWindowThreads] =     new CheckboxMenuItem(translate("&Threads             Ctrl+Shift+T")));
    tmpwindowMenu.add(items[miWindowBreakpoints] = new CheckboxMenuItem(translate("&Breakpoints         Ctrl+Shift+B")));
    tmpwindowMenu.add(items[miWindowOutput] =      new CheckboxMenuItem(translate("&Output              Ctrl+Shift+O")));
    tmpwindowMenu.add(items[miWindowInspector] =   new CheckboxMenuItem(translate("Component &inspector Ctrl+Shift+I")));

    mb.add((Menu)(items[helpMenu] = tmphelpMenu = new Menu(translate("&Help"))));
    tmphelpMenu.add(items[miHelpAbout] = new MenuItem(translate("&About...")));

    setMenuBar(mb);
    setBackground(Color.lightGray);
  }

  void setupMenuItems()
  {
    disableItem(miSystemSave);
    disableItem(miSystemSaveTemplate);

    disableItem(miEditUndo);
    disableItem(miEditCut);
    disableItem(miEditCopy);
    disableItem(miEditPaste);
    disableItem(miEditGoTo);
    disableItem(miEditFind);

    ((CheckboxMenuItem)items[miDesignShowGrid]).setState(true);

    disableItem(miCompileMain);
    disableItem(miRunMain);
    disableItem(miRunSetMain);

    enableItem(miDebugGo);
    disableItem(miDebugGoCursor);
    disableItem(miDebugFinish);
    disableItem(miDebugRestart);
    disableItem(miDebugTraceOver);
    disableItem(miDebugTraceInto);
    disableItem(miDebugToggleBreak);

    disableItem(miWindowClose);
  }

    /** Translates string used in menu items. For other systems than MS Windows deletes all occurences
      of '&' character.
      */
  protected String translate(String item)
  {
    if((TopLevel.getOS() == TopLevel.OS_WINDOWS95 || TopLevel.getOS() == TopLevel.OS_WINDOWSNT) &&
      TopLevel.getJavaVersion() == TopLevel.JAVA_UNKNOWN) return item;
    else
    {
      StringTokenizer t = new StringTokenizer(item, "&", false);
      StringBuffer buf = new StringBuffer(40);
      while(t.hasMoreTokens()) buf.append(t.nextToken());
      return buf.toString();
    }
  }

    /** Gives access to the Items of this menu.
      @param itemId id of the required menu item or submenu.
      @return menu item reference */
  MenuItem getMenuItem(int itemId)
  {
    return items[itemId];
  }

  /** Use this to invoke the same action as if menu item was selected. It calls handleEvent with proper
    menu item in Event's target but before it tests if item is enabled.
    @param itemId id of a menu item
   */
  public void itemAction(int itemId)
  {
    MenuItem item = items[itemId];
    if(item.isEnabled())
    {
      if(item instanceof CheckboxMenuItem) ((CheckboxMenuItem)item).setState(!((CheckboxMenuItem)item).getState());
      handleEvent(new Event(item, Event.ACTION_EVENT, item.getLabel()));
    }
  }

   /** Get menu item state.
     @param itemId menu item
     @return true if the menu item is enabled
    */
  public boolean isItemEnabled(int itemId)
  {
    return items[itemId].isEnabled();
  }

   /** Disables the menu item.
     @param itemId menu item
    */
  public void disableItem(int itemId)
  {
    items[itemId].disable();
    XFToolbar tb = TopLevel.getToolbar();
    if (tb != null) tb.disableCommand(itemId);
  }

   /** Enables the menu item. It applies some extra checking for determine whether some special
     menu items can be enabled.
     @param itemId menu item to be enabled
    */
  public void enableItem(int itemId)
  {
    boolean result;
    switch(itemId)
    {
      case miDebugGoCursor: result = TopLevel.focusedEditor() != null &&
        TopLevel.getDebugger().canEnableGoToCursor();
        break;
      default: result = true;
    }
    items[itemId].enable(result);
    XFToolbar tb = TopLevel.getToolbar();
    if(tb != null) tb.enableCommand(itemId, result);
  }

   /** Conditionally enables the menu item.
     @param itemId menu item to enable
     @param condition true for enable, false for disable
    */
  public void enableItem(int itemId, boolean condition)
  {
    if(condition) enableItem(itemId);
    else disableItem(itemId);
  }

    /** Gives access to the Items of this menu.
      @param itemId id of the required menu item or submenu.
      @return menu item reference */
  public MenuItem getItem(int itemId)
  {
    return items[itemId];
  }

  public boolean action(Event event, Object arg)
  {

    if(event.target instanceof MenuItem)
    {
      // >>System
      if(event.target == items[miSystemNewClass])
      {
       new NewClassDialog(this).show();
       return true;
      }


      if(event.target == items[miSystemNewPackage])
      {
       new NewPackageDialog(this).show();
       return true;
      }

      else if(event.target == items[miSystemNewBrowser])
      {
        BrowserSettings.CreateBrowserWindow(true);
        return true;
      }

      else if(event.target == items[miSystemSave])
      {
        TopWindowBase foc = TopLevel.getFocusedBase();
        if(foc.id == TopLevel.W_EDITOR)
          saveSource(((JavaEditorFrame)foc.getWindow()).getSource());
        else if(foc.id == TopLevel.W_FORM_FRAME || foc.id == TopLevel.W_FORM_DIALOG ||
          foc.id == TopLevel.W_FORM_APPLET)
          saveSource(((XelfiDesignForm)foc.getWindow()).getSource());
        return true;
      }

      else if(event.target == items[miSystemSaveAll])
      {
        try
        {
          Editor.saveAll();
        }
        catch(IOException e) { new MessageBox("Error", "IOException while saving").show(); }
        return true;
      }

      else if(event.target == items[miSystemSaveTemplate])
      {
        TopWindowBase foc = TopLevel.getFocusedBase();
        if(foc.id == TopLevel.W_EDITOR)
          new SaveAsTemplDialog(this, ((JavaEditorFrame)foc.getWindow()).getSource()).show();
        else if(foc.id == TopLevel.W_FORM_FRAME || foc.id == TopLevel.W_FORM_DIALOG ||
          foc.id == TopLevel.W_FORM_APPLET)
          new SaveAsTemplDialog(this, ((XelfiDesignForm)foc.getWindow()).getSource()).show();
        return true;
      }

      else if(event.target == items[miSystemOptions])
      {
        new OptionsDialog(TopLevel.getOptions()).show();
        return true;
      }

      else if(event.target == items[miSystemMemory])
      {
        new MemoryView().show();
        return true;
      }

      else if(event.target == items[miSystemExit])
      {
        TopLevel.exitXelfi();
        return true;
      }
      // <<System

      // Edit menu>>
      else if(event.target == items[miEditCut])
      {
        JavaEditorFrame f = TopLevel.focusedEditor();
        try {
          xelfi.editor.TextEditorOperation.cut (f.getTextEditor());
        } catch (NotAllowedException ex) {
        };
        return true;
      }

      else if(event.target == items[miEditCopy])
      {
        JavaEditorFrame f = TopLevel.focusedEditor();
        xelfi.editor.TextEditorOperation.copy (f.getTextEditor());
        return true;
      }

      else if(event.target == items[miEditPaste])
      {
        JavaEditorFrame f = TopLevel.focusedEditor();
        try {
          xelfi.editor.TextEditorOperation.paste (f.getTextEditor());
        } catch (NotAllowedException ex) {
        };
        return true;
      }

      else if(event.target == items[miEditUndo])
      {
        JavaEditorFrame f = TopLevel.focusedEditor();
        f.getTextEditor().undo();
        return true;
      }

      else if(event.target == items[miEditFind])
      {
        JavaEditorFrame f = TopLevel.focusedEditor();
        f.showFind();
        return true;
      }

      else if(event.target == items[miEditGoTo])
      {
        JavaEditorFrame f = TopLevel.focusedEditor();
        f.showGoTo();
        return true;
      }
      // <<Edit

      if(event.target == items[miDesignRealMode])
      {
        TopLevel.switchRealMode(((CheckboxMenuItem)event.target).getState());
      }

      else if(event.target == items[miDesignComponentsOrder])

      {

       // [now]
       XelfiDesignInspectable xi = TopLevel.getInspector().getInspectable();
       if (xi != null) {
           Vector selCs = xi.getSelectedComponents();
           if ((selCs.size() == 1) && (selCs.elementAt(0) instanceof XelfiVisualContainer))
           new xelfi.design.components.dialogs.XFOrderDialog(this, (XelfiVisualContainer)selCs.elementAt(0)).show();
       }
      }

      else if(event.target == items[miDesignShowGrid])
      {
        TopLevel.showGrid(((CheckboxMenuItem)event.target).getState());
      }

      else if(event.target == items[miDesignLockComponents])
      {
       XFDesignOptions.lockComponents = ((CheckboxMenuItem)event.target).getState();
      }

      else if(event.target == items[miDesignSelectionMode])
      {
        TopLevel.getPalette().setSelectionMode();
      }

      else if(event.target == items[miDesignPaletteManager])
      {
        new ComponentPaletteEditor(this).show();
      }

      else if(event.target == items[miDesignDock])
      {
        actionPaletteDockSwitch();
        return true;
      }
      // <<Design

      // >> Run
      else if(event.target == items[miCompile])
      {
        Vector focused = TopLevel.getFocusedClasses();
        if(focused.size() == 0)
        {
          String main = RunOptions.getMainClass();
          if(main == null) new MessageBox("Error", "No class to compile.").show();
          else XelfiCompiler.compileClass(main);
        }
        else XelfiCompiler.compileClasses(focused);
        return true;
      }

      else if(event.target == items[miCompileMain])
      {
        XelfiCompiler.compileClass(RunOptions.getMainClass());
        return true;
      }

      else if(event.target == items[miCompileAll])
      {
        XelfiCompiler.compileAll();
        return true;
      }

      else if(event.target == items[miRun])
      {
        String clName = TopLevel.getFocusedClass();
        if(clName == null) clName = RunOptions.getMainClass();
        if(clName != null) Executor.execute(clName);
        else new MessageBox("Error", "No class to run.").show();
        return true;
      }

      else if(event.target == items[miRunApplet])
      {
        String clName = TopLevel.getFocusedClass();
        if(clName == null) clName = RunOptions.getMainClass();
        if(clName != null) Executor.runApplet(clName);
        else new MessageBox("Error", "No applet to run.").show();
        return true;
      }

      else if(event.target == items[miRunMain])
      {
        Executor.execute(RunOptions.getMainClass());
        return true;
      }

      else if(event.target == items[miRunSetMain])
      {
        String clName = TopLevel.getFocusedClass();
        if(clName != null) RunOptions.setMainClass(clName);
        return true;
      }
      // <<Run

      // Debug>>
      else if (event.target == items[miDebugGo])
      {
        TopLevel.getDebugger().go();
      }

      else if (event.target == items[miDebugGoCursor])
      {
        JavaEditorFrame ed = TopLevel.focusedEditor();
        if(ed == null) new MessageBox("Error", "Go to cursor error: no editor is focused").show();
        else TopLevel.getDebugger().goToCursor(ed.getSource(), ed.getTextEditor().getCursorPos());
      }

      else if (event.target == items[miDebugFinish])
      {
        TopLevel.getDebugger().finishDebugging();
      }

      else if (event.target == items[miDebugRestart])
      {
        TopLevel.getDebugger().restartDebugging();
      }

      else if (event.target == items[miDebugTraceOver])
      {
        TopLevel.getDebugger().traceOver();
      }

      else if (event.target == items[miDebugTraceInto])
      {
        TopLevel.getDebugger().traceInto();
      }

      else if (event.target == items[miDebugToggleBreak])
      {
        JavaEditorFrame ed = TopLevel.focusedEditor();
        if(ed == null) new MessageBox("Error", "Toggle breakpoint error: no editor is focused").show();
        else TopLevel.getDebugger().toggleBreakpoint(ed.getSource(), ed.getTextEditor().getCursorPos());
      }

      else if (event.target == items[miDebugAddWatch])
      {
        TopLevel.getDebugger().addWatch();
      }
      // <<Debug

        // Window>>
      else if (event.target == items[miWindowWatches])
      {
        WatchesView view = TopLevel.getDebugger().getWatches().getView();

        TopLevel.bringToTop(view, true);
        ((CheckboxMenuItem)items[miWindowWatches]).setState(true);
        return true;
      }

      else if (event.target == items[miWindowLocals])
      {
        TopLevel.bringToTop(TopLevel.getDebugger().getLocalsView(), true);
        ((CheckboxMenuItem)items[miWindowLocals]).setState(true);
        return true;
      }

      else if (event.target == items[miWindowCallStack])
      {
        TopLevel.bringToTop(TopLevel.getDebugger().getCallStackView(), true);
        ((CheckboxMenuItem)items[miWindowCallStack]).setState(true);
        return true;
      }

      else if (event.target == items[miWindowThreads])
      {
        TopLevel.bringToTop(TopLevel.getDebugger().getThreadsView(), true);
        ((CheckboxMenuItem)items[miWindowThreads]).setState(true);
        return true;
      }

      else if (event.target == items[miWindowBreakpoints])
      {
        TopLevel.bringToTop(TopLevel.getDebugger().getBreakpoints().getView(), true);
        ((CheckboxMenuItem)items[miWindowBreakpoints]).setState(true);
        return true;
      }

      else if(event.target == items[miWindowOutput])
      {
        TopLevel.bringToTop(TopLevel.getOutput(), true);
        ((CheckboxMenuItem)items[miWindowOutput]).setState(true);
        return true;
      }

      else if(event.target == items[miWindowInspector])
      {
        TopLevel.bringToTop(((XFComponentInspector)TopLevel.getInspector()), true);
        ((CheckboxMenuItem)items[miWindowInspector]).setState(true);
        return true;
      }

      else if(event.target == items[miWindowClose])
      {
        TopLevel.getFocused().handleEvent(new Event(TopLevel.getFocused(), Event.WINDOW_DESTROY, null));
        return true;
      }

      else if(event.target == items[miWindowCloseAll])
      {
        Desktop.closeAllWindows();
        return true;
      }
      // <<Window

      // >>Help
      else if(event.target == items[miHelpAbout])
      {
        new AboutBox(this).show();
        return true;
      }
      // <<Help

        // check for EditorMenuItem
      else if(event.target instanceof EditorMenuItem)
      {
        EditorMenuItem i = (EditorMenuItem)event.target;
        i.selected();
      }
        // Menu item design/addmode is not controlled by this MainMenu. XFComponentPalette is expected
        //  to set it, and handle it's item actions (in its handleEvent method)
      else if(((XFComponentPalette)TopLevel.getPalette()).handleEvent(event)) return true;
      else
      {
        new MessageBox("Warning", "Selected menu item is not supported yet.").show();
        return false;
      }
    // end instanceof MenuItem
    }
    return super.action(event, arg);
  }

/*  protected void actionRun(String clName)
  {
    Executor.execute(clName);
  }
*/

  private void actionPaletteDockSwitch()
  {
    XFComponentPalette pal = (XFComponentPalette)TopLevel.getPalette();
    XFToolbar tb = TopLevel.getToolbar();

    if(pal.getParent() == this)
    {
      remove(pal);
      tb.setRows(1);
      resize(Toolkit.getDefaultToolkit().getScreenSize().width, preferredSize().height);
      validate();
      PaletteWindow w = new PaletteWindow(this, pal);
      w.show();
      ((CheckboxMenuItem)items[miDesignDock]).setState(false);
    }
    else
    {
      Window p = (Window)pal.getParent();
      p.remove(pal);
      p.hide();
      p.dispose();
      add("Center", pal);
      ((CheckboxMenuItem)items[miDesignDock]).setState(true);
      tb.setRows(2);
      resize(Toolkit.getDefaultToolkit().getScreenSize().width, preferredSize().height);
      validate();
    }
  }

    // handles IOException when saving
  private void saveSource(Source src)
  {
    try
    {
      src.save();
    }
    catch(IOException e)
    {
      new MessageBox("Error", "Error when saving source for class " + src.getName() +
        " (" + e + ")");
    }
  }

  public boolean handleEvent(Event event)
  {
    if(event.target == this && event.id == Event.WINDOW_DESTROY)
    {
      TopLevel.exitXelfi();
      return true;
    }
    return super.handleEvent(event);
  }

  public void show()
  {
//    pack();
//    move(0, 0);
//    resize(Toolkit.getDefaultToolkit().getScreenSize().width, preferredSize().height);
    super.show();
  }
}






