/**  Top level desktop dialog
* @author Filip Dvorak
* @version 2.0
*/

package xelfi.top;

import java.awt.*;
//import java.util.*;

//import xelfi.design.forms.XelfiDesignForm;
//import xelfi.compiler.Output;

  /** Inherit this class to create a xelfi top level desktop dialog window.
   */
public class TopDialog extends Dialog
{
  private TopWindowBase base;

  /** Calls matching constructor of Dialog. Parent will be xelfi main window.*/
  public TopDialog(String title, boolean modal)
  {
    super(TopLevel.getMenu(), title, modal);
    base = new TopWindowBase(this);
  }

  /** Calls matching constructor of Dialog. */
  public TopDialog(Frame parent, boolean modal)
  {
    super(parent, modal);
    base = new TopWindowBase(this);
  }

  /** Calls matching constructor of Dialog. */
  public TopDialog(Frame parent, String title, boolean modal)
  {
    super(parent, title, modal);
    base = new TopWindowBase(this);
  }

  public boolean handleEvent(Event ev)
  {
    if(base.handleEvent(ev)) return true;
    else return super.handleEvent(ev);
  }

    /** Handles Window menu items updating, setting window positions, etc. */
  public void show()
  {
    base.beforeShow();
    // Thread.currentThread().setPriority(Thread.MIN_PRIORITY);
    super.show();
  }

    /** Handles Window menu items updating. */
  public void hide()
  {
    super.hide();
    base.afterHide();
  }

  /** Handles focus switchinhg. The system needs to know the window that has got the focus. */
  public boolean gotFocus(Event ev, Object what)
  {
    boolean bas = base.gotFocus(ev, what);
    boolean par = super.gotFocus(ev, what);
    return bas || par;
  }

  /** Handles focus switchinhg. The system needs to know the window that has got the focus. */
  public boolean lostFocus(Event ev, Object what)
  {
    boolean bas = base.lostFocus(ev, what);
    boolean par = super.lostFocus(ev, what);
    return bas || par;
  }

    /** Key shortcuts */
  public boolean keyDown(Event  e, int  key)
  {
    if(base.keyDown(e, key)) return true;
    else return super.keyDown(e, key);
  }

    /** Should be called before closing the frame is no longer needed. */
  public void dispose()
  {
    base.done();
    super.dispose();
  }

    /** Adds item to the list of focused items for this frame. Focused items are items that will be
      automatically enabled/disabled when the frame receives/looses a focus.
      @param itemId new focused item. */
  public void addFocusedItem(int itemId)
  {
    base.addFocusedItem(itemId);
  }

    /** Removes item from the list of focused items for this frame. Focused items are items that will be
      automatically enabled/disabled when the frame receives/looses a focus.
      @param itemId focused item to remove */
  public void removeFocusedItem(int itemId)
  {
    base.removeFocusedItem(itemId);
  }

  protected TopWindowBase getBase()
  {
    return base;
  }
}


