/** Save as template Dialog
* @author: Filip Dvorak
* @version: 1.0
*/

package xelfi.top.visual;

import java.awt.*;

import xelfi.templates.*;
import xelfi.editor.Source;

public class SaveAsTemplDialog extends Dialog
{
  TextField tempname;
  TemplateView templates;
  Button butOK, butCanc;
  Source templSource;


  public SaveAsTemplDialog(Frame par, Source src)
  {
    super(par, "New template", true);

    templates = new TemplateView();
    templates.setCurrent(Template.identifySource(src));
    templSource = src;

    RowLayout dl = new RowLayout();
    setLayout(dl);
    Insets di1 = new Insets(2, 2, 2, 6);
    Insets di2 = new Insets(0, 8, 4, 6);
    RowLayoutProperties dp = new RowLayoutProperties(di1, 0, 0.02f, 0);
    dp.leftSpaceGrowth = dp.rightSpaceGrowth = 0.03f;

      // parent template
    Label tf = new Label("Parent template");
    add(tf);
    dl.add(tf, dp);
    dp.compInsets = di2;
    dp.topSpaceGrowth = 0;
    dp.compGrowth = 0.5f;
    dp.bottomSpaceGrowth = 0.03f;
    add(templates);
    dl.add(templates, dp);
    dp.compGrowth = 0;

      // template name
    tf = new Label("Template name");
    dp.topSpaceGrowth = 0;
    dp.bottomSpaceGrowth = 0;
    dp.compInsets = di1;
    add(tf);
    dl.add(tf, dp);
    dp.compInsets = di2;
    dp.bottomSpaceGrowth = 0.32f;
    tempname = new TextField();
    add(tempname);
    dl.add(tempname, dp);

      // button panel
    Panel bp = new Panel();
    dp.topSpaceGrowth = 0.06f;
    dp.bottomSpaceGrowth = 0.05f;
    dp.compGrowth = 0.02f;
    dp.compInsets = new Insets(6, 2, 3, 2);
    add(bp);
    dl.add(bp, dp);

      // button panel setup
    ColLayout bpl = new ColLayout();
    ColLayoutProperties bpp = new ColLayoutProperties(new Insets(0, 2, 0, 2), 0.02f);
    bpp.leftSpaceGrowth = 0.41f;
    bpp.rightSpaceGrowth = 0.03f;
    butOK = new Button(" OK ");
    bp.add(butOK);
    bpl.add(butOK, bpp);
    bpp.compGrowth = 0.03f;
    bpp.leftSpaceGrowth = 0;
    bpp.rightSpaceGrowth = 0.41f;
    butCanc = new Button("Cancel");
    bp.add(butCanc);
    bpl.add(butCanc, bpp);

    pack();
  }

  public boolean action(Event e, Object arg)
  {
    if(e.target == butOK)
    {
      saveAsTempl();
      return true;
    }
    else if(e.target == butCanc)
    {
      hide();
      dispose();
      return true;
    }
    return super.action(e, arg);
  }

  public void show()
  {
    if(getPeer() == null) addNotify();
    Dimension scr = Toolkit.getDefaultToolkit().getScreenSize(), me = size();

    move((scr.width - me.width) / 2, (int)((scr.height - me.height) / 2.4));
    super.show();
  }

  public boolean handleEvent(Event evt)
  {
    switch(evt.id)
    {
      case Event.WINDOW_DESTROY:
        if(evt.target == this)
        {
          hide();
          dispose();
          return true;
        }
    }
    return super.handleEvent(evt);
  }

  protected void saveAsTempl()
  {
    String name = tempname.getText();
    if(name.equals("")) return;

    try
    {
      Template.newTemplate(name, templSource);
    }
    catch (Exception ex) { System.err.println(ex); }

    hide();
    dispose();
  }

  public Dimension preferredSize()
  {
    Dimension pref = super.preferredSize();
    if(pref.width < 260) pref.width = 260;
    if(pref.height > 500) pref.height = 500;
    return pref;
  }

  public Insets insets()   // Under JDK 1.0.2 for win95 insets doesnt work. This patch should help.
  {
    Insets i = super.insets();
    if(getPeer() != null && (i.top < 0 || i.bottom < 0 || i.left < 0 || i.right < 0))
      return new Insets(28, 4, 4, 4);
    else return i;
  }
}
