/** PathScanner - scanning paths utility class
* @author: Filip Dvorak
* @version: 1.0
*/

package xelfi.util;

import java.io.*;
import java.util.*;

public class PathScanner
{
    /** Scans the path string, which contains set of directoties in a common format (like classpath
      or path system property), and appends a name into each directory. If it finds some existin object
      (file , directory) returns its complete name, or returns null if it finds nothing.
     */
  public static String resolveName(String name, String path)
  {
    StringTokenizer tokenizer = new StringTokenizer(path, File.pathSeparator);

    while(tokenizer.hasMoreElements())
    {
      String dir = tokenizer.nextToken();
      dir = addFileSeparator(dir);
      String testName = dir + name;
        // test if this file name exists 
      File test = new File(testName);
      if(test.exists()) return testName;
    }
    return null;
  }

    /** This safely adds a file separator string at the end of the parameter, but testing if
      it already doesn't end with the separator - so for ex. if you call it with root dir "/"
      as a parameter you will get "/".
     */    
  public static String addFileSeparator(String arg)
  {
      // test on special case .. the root directory "/"
    String filesep = File.separator;
    if(arg.regionMatches(arg.length() - filesep.length(), filesep, 0, filesep.length()))
      return arg;
    else return arg + filesep;
  }

    /** Returns enumeration of elements of the path. Elements of enumeration are String class. 
     */
  public static Enumeration elements(String path)
  {
    return new StringTokenizer(path, File.pathSeparator);
  }
}
    

