/**
* Xelfi programming environment
*/

package xelfi.xedit;
import xelfi.editor.EditorLoader;
import xelfi.editor.Source;
import java.io.File;
import java.io.IOException;

/**
* Watches over sources. If a modified one is garbage collected
* a dialog box opens and users is asked whether to save or not.
*
* @version 1.0
* @author Jaroslav Tulach
*/

class SourceSaver extends Object implements EditorLoader {
  /** This method is called when a new document is loaded into memory.
  * In this class does nothing.
  * @param s the source object of the class
  */
  public void newDocumentHasBeenLoaded (Source s) {
    // adds a comment manager to this class
    new Comments (s.getDocument ());
  };

  /** This method is called when an old document is about to close.
  * @param s the source object of the class
  * @warning the source object s will became invalid after finishing this
  * method. So <EM>do not store pointer to it anywhere!</EM>
  * If you do not want to relase the document from memory create new
  * Source object for class s.getName ().
  * But again <EM>do not use the s object itself</EM>.
  */
  public synchronized void documentIsAboutToClose (Source s) {
    try {
      s.save ();
    } catch (IOException ex) {
      System.err.println ("Cannot save class " + s.getName () + " to file " + s.getSource ());
    };
  };
};


